/**
 * sage_layer.h
 * 
 * SAGE (Small Amiga Game Engine) project
 * Layer management
 * 
 * @author Fabrice Labrador <fabrice.labrador@gmail.com>
 * @version 1.2 November 2020
 */

#ifndef _SAGE_LAYER_H_
#define _SAGE_LAYER_H_

#include <exec/exec.h>

#include "sage_bitmap.h"
#include "sage_picture.h"

#define SLAY_MAX_LAYERS       8

#define SLAY_OVERNONE         0
#define SLAY_OVERWIDTH        1
#define SLAY_OVERHEIGHT       2
#define SLAY_OVERBOTH         3

/** SAGE layer view structure */
typedef struct {
  /** View dimension */
  ULONG left, top, width, height;
} SAGE_LayerView;

/** SAGE layer structure */
typedef struct {
  /** Layer views */
  SAGE_LayerView view[4];
  /** Layer view overflow */
  UWORD overflow;
  /** Layer bitmap */
  SAGE_Bitmap * bitmap;
} SAGE_Layer;

/** Create a layer */
BOOL SAGE_CreateLayer(UWORD, ULONG, ULONG);

/** Create a layer from a picture */
BOOL SAGE_CreateLayerFromPicture(UWORD, SAGE_Picture *);

/** Get a layer by his index */
SAGE_Layer * SAGE_GetLayer(UWORD);

/** Release a layer resources */
BOOL SAGE_ReleaseLayer(UWORD);

/** Set the layer view */
BOOL SAGE_SetLayerView(UWORD, ULONG, ULONG, ULONG, ULONG);

/** Set the layer transparency */
BOOL SAGE_SetLayerTransparency(UWORD, ULONG);

/** Get the layer bitmap */
SAGE_Bitmap * SAGE_GetLayerBitmap(UWORD);

/** Blit a picture to layer at given position */
BOOL SAGE_BlitPictureToLayer(SAGE_Picture *, ULONG, ULONG, ULONG, ULONG, UWORD, ULONG, ULONG);

/** Blit a layer to the screen at given position */
BOOL SAGE_BlitLayerToScreen(UWORD, ULONG, ULONG);

#endif
